part of 'table_widget.dart';

class _AppTableFilterRow extends StatelessWidget {
  final AppButtonConfig? headerAction;
  final List<Widget> filters;

  const _AppTableFilterRow({
    this.headerAction,
    this.filters = const [],
  });

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 46,
      padding: const EdgeInsets.symmetric(horizontal: 16),
      child: Row(
        children: <Widget>[
          ...filters,
          const Spacer(),
          if (headerAction != null) AppButton.fromConfig(headerAction!),
        ].addSpacingBetween(),
      ),
    );
  }
}

sealed class AppFilter extends StatefulWidget {
  const AppFilter({super.key});

  @override
  State<AppFilter> createState();
}

sealed class _AppFilterState extends State<AppFilter> {
  @override
  Widget build(BuildContext context) {
    return const Placeholder();
  }
}

class AppTextFilter extends AppFilter {
  final String label;
  final ValueChanged<String> onChanged;
  final String? initialValue;

  const AppTextFilter({
    required this.label,
    required this.onChanged,
    this.initialValue,
    super.key,
  });

  @override
  State<AppFilter> createState() => _AppTextFilterState();
}

class _AppTextFilterState extends _AppFilterState {
  @override
  AppTextFilter get widget => super.widget as AppTextFilter;

  late final _textController = TextEditingController(
    text: widget.initialValue ?? '',
  );

  final _overlayController = OverlayPortalController();

  final _link = LayerLink();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return CompositedTransformTarget(
      link: _link,
      child: OverlayPortal(
        controller: _overlayController,
        overlayChildBuilder: (context) {
          return CompositedTransformFollower(
            link: _link,
            showWhenUnlinked: false,
            targetAnchor: Alignment.bottomLeft,
            child: Align(
              alignment: Alignment.topLeft,
              child: SizedBox(
                width: 300,
                child: TapRegion(
                  onTapOutside: (_) => _overlayController.hide(),
                  child: Card(
                    child: Padding(
                      padding: const EdgeInsets.all(8),
                      child: Column(
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          TextFormField(
                            controller: _textController,
                            onChanged: widget.onChanged,
                            decoration: InputDecoration(
                              labelText: widget.label,
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
                ),
              ),
            ),
          );
        },
        child: ListenableBuilder(
          listenable: _textController,
          builder: (context, child) {
            return FilterChip(
              label: Text(_textController.text.isEmpty ? widget.label : _textController.text),
              onSelected: (_) => _overlayController.show(),
            );
          },
        ),
      ),
    );
  }
}
